<?php
/**
* Analyses the parsing result. 
* 
* Task of the PhpdocAnalyser is to interpret the data returned by the parser.
* It looks for inherited and overriden functions and variables, "brothers" and
* build a class tree.
*
* @author	 Ulf Wendel <ulf@redsys.de>
* @version 0.2alpha
* @since 	 0.2alpha
*/
class PhpdocParsingAnalyser extends PhpdocAnalyserObject {

	/**
	* Array of data fields of a function that brother and sisters share...
	* @var	array	$brotherFields
	* @see	findBrothersandSisters
	*/
	var $brotherFields = array (
																"shortdesc"			=> "",
																"desc"					=> "",
																"param"					=> array(),
																"return"				=> array(),
																"throws"				=> array(),
																"see"						=> array(),
																"access"				=> "",
																"version"				=> "",
																"since"					=> "",
																"abstract"			=> "",
																"static"				=> "",
																"author"				=> "",
																"copyright"			=> "",
																"ignored"				=> array(),
																"undocumented"	=> "" 
														);
	
	/**
	* Array of data fields of a function that can't be inherited or overriden.
	* @var	array	$notInheritedFunctionFields
	* @see	updateFunctionFields()
	*/
	var $notInheritedFunctionFields = array(	
																						"inherited" 		=> true,
																						"overriden"			=> true,
																						"name"					=> true,
																						"undocumented"	=> true,
																						"baseclass"			=> true,
																						"ignored"				=> true
																				);

	/**
	* Array of data fields of a variable that can't be inherited or overriden.
	* @var	array	notInheritedVariableFields
	* @see	updateVariableFields()
	*/																				
	var $notInheritedVariableFields = array( 	
																						"inherited"			=> true,
																						"overriden"			=> true,
																						"name"					=> true,
																						"undocumented"	=> true,
																						"baseclass"			=> true,
																						"ignored"				=> true
																					);	
																					

	/**
	* Sets the output flag. If true the anamyser prints some status messages
	* @param	boolean	$flag_output	(optional, default: false)
	* @see		flag_output
	*/
	function PhpdocParsingAnalyser($flag_output = false) {
		if ($flag_output)
			$this->setFlagOutput(true);
		else 
			$this->setFlagOutput(false);
	}																																		
	
	/**
	* Starts the analyser
	* @return	array			$classes			Array of classes with new data
	* @access	public
	*/
	function analyse() {
	
		$this->outl("   ... building class tree");
		$this->buildClasstree();
		
		$this->outl("   ... updating using the new informations from the class tree");
		$this->findInheritedandOverriden();
		$this->updateAccessReturn();
		
		$this->outl("   ... sorting data by their 'type' (public, private, undocumented)");
		$this->findPublicPrivate();
		
		$this->outl("   ... checking for brothers and sisters");
		$this->findBrothersandSisters();
		
		return array($this->classTree, $this->classes);		
	} // end func analyse
	
	/**
	* Interprets brother and sister tags
	*/
	function findBrothersAndSisters() {
	
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {
		
				reset($this->classes[$filename][$classname]["functions"]);
				while (list($functionname, $data)=each($this->classes[$filename][$classname]["functions"])) {
					$brother = $data["brother"];
					if (""==$brother)
						continue;
	
					if ( isset($data["overriden"]) || isset($data["inherited"]) ) {
						// can't use brother on inherited functions
						$this->classes[$filename][$classname]["functions"][$functionname]["brother"] = "";
						$this->classes[$filename][$classname]["functions"][$functionname]["ignored"][] = $brother;
						continue;
					}

					$brotherdata = array();
					$parentclass = "";
					
					if (isset($this->classes[$filename][$classname]["functions"][$brother])) {
					
						$brotherdata = $this->classes[$filename][$classname]["functions"][$brother];
						
					} else {
					
						$path = $this->classes[$filename][$classname]["path"];
						reset($path);
						while (list($k, $parentclass)=each($path)) {
							$parentfilename = $this->classnamesToFilenames[$parentclass];
							if (isset($this->classes[$parentfilename][$parentclass]["functions"][$brother])) {
								$brotherdata = $this->classes[$parentfilename][$parentclass]["functions"][$brother];
								break;
							}
						}
							
					}
					
					if (0!=count($brotherdata)) {

						reset($brotherdata);
						while (list($field, $value)=each($brotherdata)) {
							if (0==$data[$field] || 0==count($data[$field]))
								$data[$field] = $value;
						}				
						
						$data["brother"] = array ( 
																				"parentclass"	=> $parentclass,
																				"function"		=> $brother
																			);
																			
					} else {
						
						$data["ignored"][] = $data["brother"];
						
					}
					
					$this->classes[$filename][$classname]["functions"][$functionname] = $data;

				}
				
		}
		
	} // end func findBrothersAndSisters
	
	/**
	* Builds the classtree, computes the "path" and the direct known subclasses
	* @access	private
	*/	
	function buildClasstree() {

		// work on all classes...
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {
			$this->classes[$filename][$classname]["filename"] = $filename;

			$path 					= array();
			$baseclass 			= $classname;
			$basefile	 			= $filename;
			$flag_noparent	= false;
			
			// look for parents
			while ( $extends = $this->classes[$basefile][$baseclass]["extends"] ) {
				if (!isset($this->classnamesToFilenames[$extends])) {
					$flag_noparent = true;
					break;
				}
					
				$path[] 		= $extends;
				$baseclass 	= $extends;
				$basefile		= $this->classnamesToFilenames[$baseclass];
			} 
			$this->classes[$filename][$classname]["path"] = $path;
			$this->classes[$filename][$classname]["warnings"]["noparent"] = $flag_noparent;
			/*
			if (!isset($this->classes[$basefile][$baseclass])) {
				$this->classes[$filename][$classname]["warnings"]["noparent"] = true;
			} else {
				$this->classes[$filename][$classname]["warnings"]["noparent"] = false;
			}
			*/
			
			$base = (0==count($path)) ? true : false;
			
			$this->classTree[$classname]["base"] = $base;
			if ($baseclass != $classname)
				$this->classes[$filename][$classname]["baseclass"] = $baseclass;
			$this->classTree[$classname]["path"] = $path;
			
		} // end while this->classnamesToFilenames
		
		// walk through the tree and find the subclasses
		reset($this->classTree); 
		while (list($classname, $tree)=each($this->classTree)) {
			// skip base classes
			if (!is_array($tree["path"])) 
				continue;
			
			// build a list of subclasses	
			reset($tree["path"]);
			while (list($k, $baseclass)=each($tree["path"])) {
				$this->classTree[$baseclass]["subclasses"][$classname] = true;
				$classname = $baseclass;
			}
		} 
		
		ksort($this->classTree);
	} // end func buildClasstree
	
	/**
	* Looks for inherited and overriden functions and variables
	* and copies them into the class list array
	* @access	private
	* @see		findInheritedVariables()
	*/
	function findInheritedandOverriden() {
		
		$functiontree = array();
		$variabletree = array();
		
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {

			$this->classes[$filename][$classname]["inherited"]["functions"] = array();
			$this->classes[$filename][$classname]["inherited"]["variables"] = array();
			$this->classes[$filename][$classname]["overriden"]["functions"] = array();
			$this->classes[$filename][$classname]["overriden"]["variables"] = array();
		
			$tree = $this->classTree[$classname];
			// skip base classes
			if (true == $tree["base"]) 
				continue;
			
			reset($tree["path"]);
			while (list($k, $parentclass)=each($tree["path"])) {
			
				$parentfilename		= $this->classnamesToFilenames[$parentclass];
				$parentfunctions	= $this->classes[$parentfilename][$parentclass]["functions"];
				$parentvariables	= $this->classes[$parentfilename][$parentclass]["variables"];
				
				// look for inherited and overriden functions
				if (is_array($parentfunctions)) {
				
					// can't override or inherit a constructor...
					unset($parentfunctions[$parentclass]);
					
					reset($parentfunctions);
					while (list($functionname, $data)=each($parentfunctions)) {
					
						if (!isset($this->classes[$filename][$classname]["functions"][$functionname])) {
						
							$this->classes[$filename][$classname]["inherited"]["functions"][$parentclass][$functionname] = true;
							
						} else {
						
							$this->classes[$filename][$classname]["overriden"]["functions"][$parentclass][$functionname] = true;
							$this->classes[$filename][$classname]["functions"][$functionname]["baseclass"] = $parentclass;
							if (false == $this->classes[$parentfilename][$parentclass]["functions"][$functionname]["undocumented"]) 
								$functiontree[$filename][$classname][$functionname][] = $parentclass;
								
						}
						
					} // end while $parentfunctions
					
				} // end if is_array($parentfunctions)

				// look for inherited and overriden variables
				if (is_array($parentvariables)) {
				
					reset($parentvariables);
					while (list($varname, $data)=each($parentvariables)) {
					
						if (!isset($this->classes[$filename][$classname]["variables"][$varname])) {
						
							$this->classes[$filename][$classname]["inherited"]["variables"][$parentclass][$varname] = true;
							
						} else {
						
							$this->classes[$filename][$classname]["overriden"]["variables"][$parentclass][$varname] = true;
							$this->classes[$filename][$classname]["variables"][$varname]["baseclass"] = $parentclass;
							if ( false == $this->classes[$parentfilename][$parentclass]["variables"][$varname]["undocumented"])
								$variabletree[$filename][$classname][$varname][] = $parentclass;
								
						}
						
					} // end while $parentvariables
					 
				} // end if is_array($parentvariables)
							
			} // end while %tree["path"]
				
		} // end while $this->classnamesToFilenames

		// now that we know the overriden functions and variables
		// we can copy some inherited data to the classes and 
		// build a list of overriden functions
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {
		
			if (isset($functiontree[$filename][$classname])) 
				$this->updateInheritedandOverridenFields($functiontree[$filename][$classname], $filename, $classname, "functions");
				
			if (isset($variabletree[$filename][$classname]))
				$this->updateInheritedandOverridenFields($variabletree[$filename][$classname], $filename, $classname, "variables");
		} 
		
	} // end func findInheritedandOverriden
	
	/**
	* Copies inherited and overriden data fields of functions or variables
	* @param	array		$tree
	* @param	string	$filename
	* @param	string	$classname
	* @param	enum		$type					allowed: "variables", "functions" (default: functions)
	* @see		findInheritedanOverriden(), notInheritedFunctionFields, 
	*					updateInheritedandOverridenFields()
	*/
	function updateInheritedandOverridenFields($tree, $filename, $classname, $type) {
	
		if ("variables" == $type)
			$notInherited = $this->notInheritedVariableFields;
		else {
			$type = "functions";
			$notInherited = $this->notInheritedFunctionFields;
		}
		
		#$this->introspection("class", $this->classes[$filename]);
		reset($tree);
		
		while (list($elname, $path)=each($tree)) {
		
			$currentdata = $this->classes[$filename][$classname][$type][$elname];
			
			reset($path);
			while (list($k, $parentclass)=each($path)) {
				$parentfilename = $this->classnamesToFilenames[$parentclass];
				$parentdata = $this->classes[$parentfilename][$parentclass][$type][$elname];
				if ( true == $parentdata["undocumented"] )
					continue;

				$currentdata["undocumented"] = false;
				reset($parentdata);
				while (list($field, $v)=each($parentdata)) {
					if (isset($notInherited[$field]))
						continue;
					
					$copied_before = "";
					if (isset($currentdata["inherited"][$field]))
						$copied_before = "inherited";
					else if (isset($currentdata["overriden"][$field]))
						$copied_before = "overrriden";
					
					if (""!=$copied_before) {
						// sometimes we compare skalar data, sometimes arrays. 
						// serialize is faster than a custom function
						if (serialize($v) == serialize($currentdata[$field])) 
							$currentdata[$copied_before][$field] = $parentclass;

					} else {
						
						if (""==$currentdata[$field] || 0==count($currentdata[$field])) {
							$currentdata[$field] = $v;
							$currentdata["inherited"][$field] = $parentclass;	
						} else {
							$currentdata["overriden"][$field] = $parentclass;
						}
						
					}
						
					// sometimes we compare skalar data, sometimes array
					// serialize does a good job for this

				}
					
			} // end while $path
			
			$this->classes[$filename][$classname][$type][$elname] = $currentdata;
			
		} // end while $tree

		
	} // end func updateInheritedOverridenFields()
	
	/**
	* Updates access and return parameters of functions and variables.
	* 
	* Phpdoc copies return and access specifications from parentclasses to
	* subclasses or looks for default values.
	* 
	* @access	private
	* @see		updateFunctions(), updateVariables
	*/
	function updateAccessReturn() {
	
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {
			$this->updateFunctions($filename, $classname);
			$this->updateVariables($filename, $classname);		
		}
		
	} // end func updateAccessReturn
	
	/**
	* Update access parameters of variables.
	* @param	string	$filename
	* @param	string	$classname
	* @access	private
	* @see	updateFunctions(), updateAccessReturn()
	*/
	function updateVariables($filename, $classname) {
	
		$variables = $this->classes[$filename][$classname]["variables"];
		if (!is_array($variables)) 
			return;
		
		reset($variables);
		while (list($varname, $data)=each($variables)) {
			if (""!=$data["access"]) 
				return;
			
			if (isset($data["baseclass"])) {
			
				$basefilename = $this->classnamesToFilenames[$data["baseclass"]];
				$access = $this->classes[$basefilename][$data["baseclass"]]["variables"][$varname]["access"];
				if (true == $this->classes[$basefilename][$data["baseclass"]]["variables"][$varname]["undocumented"]) {
					$access = "undocumented";
				} else if (""==$access) {
					$access = "private";
				}
					
			} else if (true == $data["undocumented"]) {
				$access = "undocumented";
		 	}	else  {
				$access = "private";
			}
			
			$this->classes[$filename][$classname]["variables"][$varname]["access"] = $access;
				
		}	
	} // end func updateVariables
	
	/**
	* Updates return and access parameters of functions.
	* @param	string	$filename
	* @param	string	$classname
	* @access	private 
	* @see		updateVariables(), updateAccessReturn()
	*/
	function updateFunctions($filename, $classname) {
	
		if (!isset($this->classes[$filename][$classname]["functions"]))
			return;
			
		$functions = $this->classes[$filename][$classname]["functions"];
		if (!is_array($functions))
			return;

		reset($functions);
		while (list($functionname, $data)=each($functions)) {

			if ($classname == $functionname) {
			
				// constructors are always public...
				$access = "public";
				$return = array (
														"type"	=> "void",
														"name"	=> "",
														"desc"	=> ""
												);
				
			} else {
				
				$access = $data["access"];
				$return = $data["return"];
				
				// no access specified?				
				if (""==$access) {

					if (isset($data["baseclass"])) {
						// inherited value
						$basefilename = $this->classnamesToFilenames[$data["baseclass"]];
						$access = $this->classes[$basefilename][$data["baseclass"]]["functions"][$functionname]["access"];
						if (true == $this->classes[$basefilename][$data["baseclass"]]["functions"][$functionname]["undocumented"]) {
							$access = "undocumented";
						} else if (""==$access) {
							$access = "private";
						}
						
					} else if (true == $data["undocumented"]) {
						// undocumented function
						$access = "undocumented";
			 		}	else {
						// just empty, default private
						$access = "private";
					}
					
				}	// end if ""==$access
					
				if (0 == count($return)) {
					
					$return = array (
														"type"	=> "",
														"name"	=> "",
														"desc"	=> ""
												);
											
					if (isset($data["baseclass"])) {
						$basefilename = $this->classnamesToFilenames[$data["baseclass"]];
						$data = $this->classes[$basefilename][$data["baseclass"]]["functions"][$functionname];  
					}
					
				  if (true == $data["undocumented"]) {
						$return["type"] = "undocumented";						
					} else if (0==count($data["return"])) {
						$return["type"] = "void";
					} else 
						$return = $data["return"];
					
				} // end if 0==count($return) 
				
			} // end if $filename == $classname

			$this->classes[$filename][$classname]["functions"][$functionname]["return"] = $return;		
			$this->classes[$filename][$classname]["functions"][$functionname]["access"] = $access;
			
		} // end while
		
	} // end func updateFunctions

	/**
	* Extracts public and private variables and functions of a class and copies the 
	* list to the classdata
	* @access	private
	*/
	function findPublicPrivate() {
		reset($this->classnamesToFilenames);
		while (list($classname, $filename)=each($this->classnamesToFilenames)) {
			
			$functions = $this->classes[$filename][$classname]["functions"];
			if (is_array($functions)) {
				
				$sorted = array(
													"public" 				=> array(),
													"private"				=> array(),
													"undocumented"	=> array()
											);				

				reset($functions);
				while (list($functionname, $data)=each($functions)) {
				
					if ($data["undocumented"])
						$sorted["undocumented"][] = $functionname;
					else if (""==$data["access"])
						$sorted["private"][] = $functionname;
					else 
						$sorted[$data["access"]][] = $functionname;
						
				}
				
				ksort($sorted["public"]);
				ksort($sorted["private"]);
				ksort($sorted["undocumented"]);
				$this->classes[$filename][$classname]["sorted_functions"] = $sorted;				
			} 

			$variables = array();			
			$variables = $this->classes[$filename][$classname]["variables"];
			if (is_array($variables)) {
			
				$sorted = array(
													"public" 				=> array(),
													"private"				=> array(),
													"undocumented"	=> array()
											);				

				reset($variables);
				while (list($varname, $data)=each($variables)) {
				
					if ($data["undocumented"])
						$sorted["undocumented"][] = $varname;
					else if (""==$data["access"]) 
						$sorted["private"][] = $varname;
					else
						$sorted[$data["access"]][] = $varname;
						
				}
				
				ksort($sorted["public"]);
				ksort($sorted["private"]);
				ksort($sorted["undocumented"]);
				$this->classes[$filename][$classname]["sorted_variables"] = $sorted;
			}  
			
		}
	} // end func findPublicPrivate
	
} // end class PhpdocParsingAnalyser
?>