<?php
/**
* This coordinates the parser and renderer. 
* 
* @author		Ulf Wendel <ulf@redsys.de>
* @version 	0.2alpha
*/
class Phpdoc extends PhpdocSetupHandler {

	/**
	* Warnings from the analyser
	* @var	array	$analyserWarnings
	*/
	var $analyserWarnings = array();
	
	/**
	* Result from the indexer
	* @var	array	$indexer
	* @see	render()
	*/
	var $indexer_result = array();
	
	/**
	* If set to true phpdoc uses html to format it's output.
	* @var	boolean	$flag_html
	* @see	Phpdoc()
	*/
	var $flag_html = false;
	
	/**
	* Print status messages
	*/
	var $flag_output = true;

	/**
	* Calls the command line handler if necessary or sets the flag_html to true
	* @see	handleArgv(), flag_html
	*/
	function Phpdoc() {
		global $argc, $PHP_SELF;
		
		$this->target = $PHP_SELF."apidoc/";
		
		if ($argc>1) 
			$this->handleArgv();
		else 
			$this->flag_html = true;
	} // end constructor

	
	/**
	* Starts the parser. 
	*
	* Extracts documentation, using some parser objects. 
	* The results are saved in objectvariables, 
	* a classtree is build, private and public data
	* gets isolated, inherited and overriden data is
	* computed.
	*
	* @return	bool		$ok
	* @throws	runtime
	* @access	public
	* @see		buildClassTree(), findInheritedAndOverriden(), updateAccessReturn(),
	*					findPublicPrivate(), $classes, $classTree, $classnamesToFilenames
	*/
	function parse() {
		
		// "setup" fine?
		$errors = $this->checkStatus();
		if (0!=count($errors)) {
			reset($errors);
			while (list($k, $error)=each($errors))
				$this->err = new exception_runtime($error["msg"], $error["errno"], 9, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return false;
		}

		if ($this->flag_html)
			$this->outl("<pre>");
			
		$this->outl("Parser starts...");
		
		// create some objects
		$fileHandler 		= new PhpdocFileHandler;
		$parser 				= new PhpdocParser(true);
		$analyser				= new PhpdocParsingAnalyser;
		
		reset($this->packages);
		while (list($package, $data)=each($this->packages)) {
			if (0==count($data["files"])) {
				$this->outl("... skipping package '$package': no files");
				continue;
			}
			
			$this->outl("... parsing package: '$package'");
			
			$contents = $fileHandler->get($data["files"]);
			
			$parser->setPhpSourcecodeFiles($contents);
			$this->packages[$package]["classes"]								= $parser->parse();
			$this->packages[$package]["classnamesToFilenames"]	= $parser->getClassnamesToFilenames();

			$this->outl("... analysing package: '$package'");
			
			$analyser->setClasses( $this->packages[$package]["classes"] );
			$analyser->setClassnamesToFilenames( $this->packages[$package]["classnamesToFilenames"] );
			list ( $this->packages[$package]["classTree"], $this->packages[$package]["classes"] ) = $analyser->analyse(true);
			
			$this->outl("... package finished");
		}
		#$this->introspection("pack", $this->packages);		
		$this->outl("Finished.");
				
		if ($this->flag_html)
			$this->outl("</pre>");
		return true;
	} // end func parse
	
	/**
	* Renders the PHPDoc Files. 
	* @param	string 	$targetdirectory
	* @return	bool		$ok
	* @throws	runtime_wrongparam
	* @access	public
	*/
	function render($targetdirectory="") {
		
		if (""==$targetdirectory)
			$targetdirectory = $this->target;
					
		if ("/"!=substr($targetdirectory, -1))
			$targetdirectory.= "/";
			
		$fileHandler 		= new PhpdocFileHandler;
		$indexer 		 		= new PhpdocIndexer($this->application, $this->packages, true);
		$indexrenderer	= new PhpdocIndexRenderer(array(), true);
		$indextypes = array ( "vertical", "horizontal", "tree");
		
		if ($this->flag_html) {
			$this->outl("<pre>");
			$index			= "<ul>";
		} else {
			$index = "\n\n";
		}
		
		$index_api	 = sprintf("<html>%s<head><title>%s</title></head>%s<body>%s",
															$this->CR_HTML,
															$this->application,
															$this->CR_HTML,
															$this->CR_HTML
													);
		$index_api.= '<ul><li><a href="index.html">index.html</a><p>&nbsp;</li>';		
		
		$this->outl("Renderer starts...");
		$this->outl("... generating Indexfiles");
		$indexer->index();
		$this->indexer_result = $indexer->getResult();
		$this->outl("... Indexer finished.");
				
		reset($this->packages);
		while (list($package, $data)=each($this->packages)) {
			
			$this->outl("... working on package: '$package'");
			
			$this->outl("   ... rendering index files");
			$indexrenderer->setIndexerResult($this->indexer_result[$package]);

			reset($indextypes);
			while (list($k, $type)=each($indextypes)) {

				$html = $indexrenderer->getClasstree($type);
				$fileHandler->createFile("index_classtree_$type.html", $html, $targetdirectory);
				if ($this->flag_html) {
					$index_api.= sprintf('<li><a href="index_classtree_%s.html">index_classtree_%s.html</a></li>%s',
																$type, 
																$type,
																$this->CR_HTML
														);
				}
				
				$index.= sprintf('<li><a href="%sindex_classtree_%s.html">index_classtree_%s.html</a></li>%s',
																$targetdirectory,
																$type,
																$type,
																$this->CR_HTML
														);				
				
				
			}
						
			$index_api.="<li><p>&nbsp;</li>";
			
			$this->outl("   ... finished.");
			$this->outl("   ... render classes in packages");
		
			while (list($classname, $tree)=each($data["classTree"])) {
			
				$filename				= $data["classnamesToFilenames"][$classname];
				
				### BUG REUSE INSTEAD! ###
				$classRenderer	= new PhpdocClassRenderer($classname, $data["classes"][$filename][$classname], $tree, $targetdirectory);
				
				$html = $classRenderer->render();
				
				$ok = $classRenderer->saveResultToFile();
				if (true == $ok) {
					$this->outl("   ... class '$classname' ok");
					
					if ($this->flag_html) {
						$index.= sprintf('<li><a href="%s%s.html">%s.html</a></li>%s',
																$targetdirectory,
																$classname,
																$classname,
																$this->CR_HTML
														);
					} 
					
					$index_api.= sprintf('<li><a href="%s.html">%s.html</a></li>%s',
																$classname,
																$classname,
																$this->CR_HTML
														);								
						
				} else {
					$this->outl("   ... class '$classname' error");
					if ($this->flag_html) 
						$index.= 			"<li>Error: $classname</li>".$this->CR_HTML;
					$index_api.= 	"<li>Error: $classname</li>".$this->CR_HTML;
				}
					
			}
		
			$this->outl("... finished package '$package'");	
		}
		
		if ($this->flag_html)
			$index.="</ul>";
		
		$ok = $fileHandler->createFile("index.html", $index_api, $targetdirectory);
		if (true == $ok) {
			$this->outl("Finished.".$this->CR_HTML);
			if ($this->flag_html)
				$this->outl("</pre>".$index);
				
		} else 
			$this->outl('Could not generate Overview.');

		return $ok;
	} // end func	render
	
	/**
	* Exports the parsing result as: wddx, DocBook
	* @param	enum		$format			name of the format you need ("wddx", "docbook"), default: wddx
	* @param	string	$targetFile	optional, if set name of the target file
	* @return	string	$result
	* @throws	wrong_param
	*/
	function export($format="wddx", $targetFile="") {
	
		$format = strtolower($format);
		if ("docbook"!=$format && "wddx"!=$format) {
			$this->err = new exception_runtime_wrongparam("Unknown format '$format'", 10, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return "";
		}
		
		if ($this->flag_html)
			$this->outl("<pre>");
			
		$this->outl("Exporter starts...");
		
		if ("docbook"==$format)
			$exporter = new PhpdocDocBookExporer(); 
		else 
			$exporter = new PhpdocWddxExporter();
		
		$exporter->Setup($this->classTree, $this->classnamesToFilenames, $this->classes);
		
		$exporter->setFlagOutput(true);
		if (""!=$targetFile)
			$exporter->setTargetFile($targetFile);
		
		$exporter->export();
		
		$this->outl("Finished.");
		
		if ($this->flag_html)
			$this->outl("</pre>");
			
		return $exporter->get();
	} // end func export
	
} // end class Phpdoc
?>