<?php
/**
* Parses phpcode to extract classes and their documentation.
*
* @version	0.2
* @author		Ulf Wendel, ulf@redsys.de
*/
class PhpdocClassParser extends PhpdocFunctionParser {

  /**
  * Array of all classes in the given code
	* 
	* The array is indexed by the classname.
	* See $emptyClass to see the internal structure.
	* 
  * @var    array $classes
  * @see		emptyClass
  */ 
  var $classes = array();
	
	/**
	* Default values of a class
	* @var	array	$emptyClass
	* @see	classes, analyseClassParagraph
	*
	*/
	var $emptyClass = array (
															"classname"		=> "",
															"extends"			=> "",
															"filename"		=> "",
															"baseclass"		=> "",
															"shortdesc"		=> "",
															"desc"				=> "",
															"author"			=> "",
															"copyright"		=> "",
															"version"			=> "",
															"since"				=> "",
															"abstract"		=> false,
															"static"			=> false,
															"ignored"			=> array(),
															
															"warnings"		=> array(		
																											"noparent" => false
																										)
												);	
	
	/**
	* Array of tags that are allowed in front of the class keyword
	* @var	array	$classTags
	* @see	analyseClassParagraph()
	*/
	var $classTags = array(
															"@abstract"		=> true,
															"@static"			=> true,
															"@author"			=> true,
															"@copyright"	=> true,
															"@version"		=> true,
															"@since"			=> true
											);
	/**
	* Just calls Setup()
	* @see	Setup()
	*/
	function PhpdocClassParser() {
		$this->Setup();			
	} // end constructor
	
	/**
	* Analyse the given PHPCode and Phpdoc paragraphs.
	* @param	string	$phpcode	code to analyse
	* @return	array		$classes
	* @throws runtime_wrongparam
	* @access	public
	*/
	function analyseClasses($paragraphs) {
		$this->classes = array();
		
		reset($paragraphs);
		while (list($k, $paragraph)=each($paragraphs)) 
			$this->analyseClassParagraph($paragraph);
			
		return $this->classes;	
	} // end func analyseClasses
	
	/**
	* Analyses a singe 
	*
	*/
	function analyseClassParagraph($paragraph) {
		
		$class 	= $this->emptyClass;
		
		list($classname, $extends)	= $this->getClassnamesFromRegs($paragraph["regs"]);
		$class["classname"]		= $classname;
		$class["extends"] 		= $extends;

		list($shortdesc, $fulldesc) = $this->getDescription($paragraph["phpdoc"]);
		$class["shortdesc"]	= $shortdesc;
		$class["desc"]				= $fulldesc;
		
		$tags = $this->getTags($paragraph["phpdoc"]);			
		$class = $this->analyseTags($tags, $class, $this->classTags);
		
		$this->classes[$classname] = $class;
	} // end func analyseClassParagraph
	
	/**
	* Finds classes and their PHP code in the given string.
	* 
	* Phpdoc currently works on per class base. That means analysing PHP source 
	* always starts with extracting classes and their PHP code from the 
	* given PHP source.
	* 
	* @param	string	$phpcode
	* @return	array		$classes	Array of PHP code, indexed by the classname
	* @see		getClassnamesFromRegs()
	*/
	function getClasscode($phpcode) {
		
		$php_class 				= "/".substr($this->PHP_CLASS, 2);
		$php_class_extend = "/".substr($this->PHP_CLASS_EXTEND, 2);
		$classes					= array();
		
		list($start, $regs) = $this->getNextClassStatement($phpcode);
		if ($start>-1) {

			list($name, $extends)=$this->getClassnamesFromRegs($regs);
			$end 				= 0;
			$pos				= $start;
			
			while (""!=$name) {
			
				$start 			= $pos;
				$remaining 	= substr($phpcode, $start+1);
				
				list ($pos, $regs) = $this->getNextClassStatement($remaining);				
				if ($pos > -1) {

					$end = strpos($phpcode, $regs[1]);
					$classes[$name] = substr($phpcode, $start, ($end-$start));
					list($classname, $extends)=$this->getClassnamesFromRegs($regs);
					$name			= $classname;
					
				} else {
				
					$classes[$name] = substr($phpcode, $start);
					$name = "";
					
				}
				
				$phpcode = $remaining;
			}
			
		}		
		
		return $classes;
	} // end getClasscode
	
	function getNextClassStatement($phpcode) {
		
		$php_class 				= "/".substr($this->PHP_CLASS, 2);
		$php_class_extend = "/".substr($this->PHP_CLASS_EXTEND, 2);
		$found 						= false;
		$pos_plain				= -1;
		$pos_extends			= -1;
		$regs_plain				= array();
		$regs_extends			= array();
		
		$ok = preg_match($php_class, $phpcode, $regs);
		if ($ok) {

			$found 			= true;
			$pos_plain 	= strpos($phpcode, $regs[1]);
			$regs_plain = $regs;
			
		}
		
		$ok = preg_match($php_class_extend, $phpcode, $regs);
		if ($ok) {
			
			$found 				= true;
			$pos_extends	= strpos($phpcode, $regs[1]);
			$regs_extends	= $regs;
			
		}
		
		if ($found) {
			
			if (-1 == $pos_plain ) {
			
				$result = array ( $pos_extends, $regs_extends) ;
				
			} else if ( -1 == $pos_extends ) {
			
				$result = array ( $pos_plain, $regs_plain );
				
			} else {
			
				if ($pos_plain<$pos_extends)
					$result = array ( $pos_plain, $regs_plain );
				else 
					$result = array( $pos_extends, $regs_extends );
					
			}
			
		} else {
			
			$result = array ( -1, array() );
			
		}
		
		return $result;
		
	}
	
	/**
	* Extracts the classname and the name of the extended class from a result array 
	* provided by preg_match($this->PHP_CLASS, ...) or  preg_match($this->PHP_CLASS_EXTEND, ...)
	* @param	array	$regs
	* @return array $names	use list ( $classname, $extends ) = getClassnamesFromRegs($regs) 
	* @see		PHP_CLASS, PHP_CLASS_EXTEND, Setup()
	*/
	function getClassnamesFromRegs($regs) {
		if (!is_array($regs))
			return array("", "");
	
		if (isset($regs[3]))
			return array($regs[2], $regs[3]);
		else 
			return array($regs[2], "");
	} // end func getClassnamesFromRegs
	
} // end class PhpdocClassParser
?>