<?php
/**
* This is the base class of all parser in phpdoc. 
*
* It defines several regexps and  provides functions to extract phpdoc 
* comments from the source.
*
* @author		Ulf Wendel <ulf@redsys.de>
* @version  0.1alpha
*/
class PhpdocParserObject extends PhpdocObject {

	/**
	* RegExp matching the beginning of a phpdoc comment
	* @var		string	$C_START					\/\*\*
	* @access	private
	*/
	var $C_START 	= '\/\*\*';
	
	/**
	* RegExp matching the end of a phpdoc comment
	* @var		string	$C_END						\*\/
	* @access	private
	*/
	var $C_END 		= '\*\/';
	
	/**
	* RegExp of allowed characters within a phpdoc comment
	* @var		string	$C_BLOCK					[^\/]*
	* @access	private
	*/
	var $C_BLOCK = '[^\/]*';
	
	/**
	* RegExp matching a whole phpdoc comment
	* @var		string	$C_ALL 						\/\*\*([^\/]*)\*\/
	* @access	private
	* @see		C_START, C_END, C_BLOCK
	*/
	var $C_ALL = '\/\*\*([^\/]*)\*\/';
	
	/**
	* RegExp matching a phpdoc element seperator
	* @var		string	$C_SPACE=						[ \t]+
	* @access	private
	*/
	var $C_SPACE = "[ \t]+";
	
	/**
	* RegExp matching a optional phpdoc element seperator
	* @var		string	$C_SPACE_OPTIONAL		 [ \t]*
	* @access	private
	*/
	var $C_SPACE_OPTIONAL = "[ \t]*";
	
	/**
	* RegExp matching a seperator between a phpdoc comment block
	* and other elements
	* @var		string		$C_SPACE_CR				[ \t\n\r]+
	* @access	private
	*/
	var $C_SPACE_CR = "[ \t\n\r]+";

	/**
	* RegExp matching an optional seperator between a phpdoc comment block
	* and other elements
	* @var		string	$C_SPACE_CR_OPTIONAL	[ \t\n\r]*
	* @access	private
	*/
	var $C_SPACE_CR_OPTIONAL = "[ \t\n\r]*";

	/**
	* RegExp matching a phpdoc newline.
	* @var		string	$C_BREAK						[\n\r]{1}
	* @access	private
	*/
	var $C_BREAK = "[\n\r]{1}";
	
	/**
	* RegExp matching the allowed variable types.
	* @var		string	$C_VARTYPE						(string|integer|int|long|double|float|bool|boolean|object|class|mixed|array)
	* @access	private
	*/
	var $C_VARTYPE = "(string|integer|int|long|double|float|bool|boolean|object|class|mixed|array)";
	
	/**
	* RegExp matching allowed optional object names
	*	@var		string	$C_OBJECTNAME_OPTIONAL	[a-zA-Z0-9_-]*
	* @access	private
	*/
	var $C_OBJECTNAME_OPTIONAL = "[a-zA-Z0-9_-]*";
	
	/**
	* RexExp matching a function name within @see.
	* @var		string	$C_SEE_FUNCTION					[a-zA-Z0-9_-]+\(\)
	* @access	private
	*/
	var $C_SEE_FUNCTION = '[a-zA-Z0-9_-]+\(\)';
	
	/**
	* RegExp matching a variable name within @see.
	* @var		string	$C_SEE_VAR 							[a-zA-Z0-9_-]+
	* @access	private
	*/
	var $C_SEE_VAR = "[a-zA-Z0-9_-]+";
	
	/**
	* RegExp matching the allowed @access specifications.
	* @var		string	$C_ACCESS							(private|public)
	* @access	private
	*/
	var $C_ACCESS = "(private|public)";
	
	/**
	* RegExp matching a valid PHP variable name.
	* @var		string	$PHP_VARNAME					[&]{0,1}[$]{1}[a-zA-Z0-9_-]+
	* @access	private
	*/
	var $PHP_VARNAME = '[&]{0,1}[$]{1}[a-zA-Z0-9_-]+';
	
	/**
	* RegExp matching a PHP function or class name.
	* @var		string	$PHP_FUNCTIONNAME		[a-zA-Z0-9_-]+
	* @access	private
	*/
	var $PHP_FUNCTIONNAME = "[a-zA-Z0-9_-]+";
	
	/**
	* RegExp matching a PHP class variable declaration.
	*
	* The variable is set by the constructor using other 
	* class variables such as PHP_VARNAME.
	*
	* @var		string	$PHP_VAR
	* @access	private
	* @see		PhpdocParserObject(), PHP_VARNAME
	*/
	var $PHP_VAR 			= "";
	
	/**
	* RegExp matching a PHP function declaration.
	* 
	* The variable is set by the constructor using other 
	* class variables such as PHP_FUNCTIONNAME
	* @var		string	$PHP_FUNCTION
	* @see		PhpdocParserObject(), PHP_FUNCTIONNAME
	*/
	var $PHP_FUNCTION = "";		
	
	/**
	* RegExp matching a PHP class declaration that does not extend another class.
	* 
	* The variable is set by the constructor using other 
	* class variables such as PHP_FUNCTIONNAME.
	* @var	string	$PHP_CLASS
	* @see	PhpdocParserObject(), PHP_FUNCTIONNAME
	*/
	var $PHP_CLASS		= "";
	
	/**
	* RegExp matching a PHP class declaration that does extend another class.
	* 
	* The variable is set by the constructor using other 
	* class variables such as PHP_FUNCTIONNAME.
	* @var	string	$PHP_CLASS_EXTEND
	* @see	PhpdocParserObject(), PHP_FUNCTIONNAME
	*/
	var $PHP_CLASS_EXTEND		= "";
	
	
	/**
	* Array of phpdoc tags, indexed by the tagname.
	*
	* @var		array		$PHPDOC_TAGS	
	* @access	private
	*/
	var $PHPDOC_TAGS = array(
														"@param" 			=> true,
														"@return" 		=> true,
														"@access"			=> true,
														"@throws"			=> true,
														"@see"				=> true,
														"@var"				=> true,
														"@brother"		=> true,
														"@version"		=> true, 
														"@author"			=> true,
														"@copyright"	=> true,
														"@since"			=> true,
														"@abstract"		=> true,
														"@static"			=> true
													);
													
	/**
	* Calls Setup().
	* 
	* Make sure that all derived classes have a constructor
	* that calls Setup, such as this one does. 
	*
	* @see	Setup()
	*/														
	function PhpdocParserObject() {
		$this->Setup();
	} // end constructor
	
	/**
	* Kind of a constructor.
	*
	* Every derived class must call this method in it's constructor!
	* Setup() sets some instance variables.
	* @see	PhpdocParserObject()
	*/													
	function Setup() {
		
		$this->PHP_VAR = sprintf("/^%s(var%s(%s)%s)/S",
													$this->C_SPACE_CR_OPTIONAL,
													$this->C_SPACE_CR,
													$this->PHP_VARNAME,
                          $this->C_SPACE_CR_OPTIONAL
												);	
												
		$this->PHP_FUNCTION = sprintf('/^%s(function%s(%s)%s\()/S',
													$this->C_SPACE_CR_OPTIONAL,
													$this->C_SPACE_CR,
													$this->PHP_FUNCTIONNAME,
                          $this->C_SPACE_CR_OPTIONAL
												);							
												
		$this->PHP_CLASS = sprintf("/^%s(class%s(%s)%s{)/S",
													$this->C_SPACE_CR_OPTIONAL,
													$this->C_SPACE_CR,
													$this->PHP_FUNCTIONNAME,
													$this->C_SPACE_CR_OPTIONAL
												);	
		
		$this->PHP_CLASS_EXTEND = sprintf("/^%s(class%s(%s)%sextends%s(%s)%s{)/S",
													$this->C_SPACE_CR_OPTIONAL,
													$this->C_SPACE_CR,
													$this->PHP_FUNCTIONNAME,
													$this->C_SPACE_CR,
													$this->C_SPACE_CR,
													$this->PHP_FUNCTIONNAME,
													$this->C_SPACE_CR_OPTIONAL
												);																												
	} // end func Setup								
					
	/**
	* Extracts Phpdoc from PHP Sourcecode
	* @param	string	$phpcode	
	* @param	enum		$keywords	
	* @return	bool		$ok
	* @see		paragraphs
	*/					
	function getPhpdocParagraphs($phpcode, $keywords="all") {
		if (""==$phpcode)
			return false;
		
		$pos	 			= 0;
		$paragraphs	= array(
													"variables"	=> array(),
													"functions"	=> array(),
													"classes"		=> array()
											);
		
		while (true) {
		
			$pos = strpos($phpcode, "/**", $pos);
			if (0==(int)$pos && "integer" != gettype($pos) ) 
				break;

      $end 				= strpos($phpcode, "*/", $pos);
			$remaining 	= substr($phpcode, $end+2);
			$found			= "";
				
			if (("all"==$keywords || "variables"==$keywords) && preg_match($this->PHP_VAR, $remaining, $regs))
				$found = "variables";
			else if (("all"==$keywords || "functions"==$keywords) &&	preg_match($this->PHP_FUNCTION, $remaining, $regs)) 
				$found = "functions";
			else if (("all"==$keywords || "classes"==$keywords) && preg_match($this->PHP_CLASS, $remaining, $regs) || preg_match($this->PHP_CLASS_EXTEND, $remaining, $regs)) 
				$found = "classes";
			if (""!=$found) {				
				$paragraph	= substr($phpcode, $pos+3, ($end-$pos)-2);
				$paragraphs[$found][] = array ( 
																				"regs"		=> $regs, 
																				"phpdoc"	=> $this->extractPhpdoc($paragraph)
																		 );
			}
			
			$pos++;
			
		} 

		return $paragraphs;
	}	// end func getPhpdocParagraphs
	/**
	* Tries to extract phpdoc comments from a code paragraph
	* @param	string	$paragraph
	* @access	private
	* @return	string	$phpdoc
	*/
	function extractPhpdoc($paragraph) {
		if (""==$paragraph)
			return "";

		$lines = split($this->C_BREAK, $paragraph);
		$phpdoc = "";

		reset($lines);
		while (list($k, $line)=each($lines)) {
			$line = trim($line);
			if ("*" == substr($line, 0, 1) && ""!=$line )		
				$phpdoc.= substr($line, 1)."\n";
		}
		
		return $phpdoc;
	} // end func extractPhpdoc
	
	/**
	* Extract the description on top of a phpdoc comment.
	* @param	string	$phpdoc
	* @return	array		$description	use list($shortdesc, $fulldesc) = getDescription($phpdoc)
	* @access	private
	*/
	function getDescription($phpdoc) {
	
		$positions = $this->getTagPos($phpdoc);
		
		if (0==count($positions)) {
			$desc = trim($phpdoc);
			$description = array ($desc, $desc);
		} else {
		
			$desc = trim(substr($phpdoc, 0, $positions[0]["pos"]));
			$lines = split($this->C_BREAK, $phpdoc);
			
			if (1==count($lines) || ""==$desc) 
				$description = array ($desc, $desc);
			else
				$description = array ( trim($lines[0]), $desc	);
		}
		
		return $description;
	} // end func getDescription
	
	/**
	* Extracts phpdoc tags from a phpdoc paragraph
	* @param	string	$phpdoc
	* @return array		$tags
	* @access	private
	* @see		getTagPos()
	*/
	function getTags($phpdoc) {
	
		$positions = $this->getTagPos($phpdoc);
		
		if (0==count($positions))
			return array();
		
		reset($positions);
		list($k, $data)=each($positions);
		$lastpos = $data["pos"];
		$lasttag = $data["tag"];
		
		while (list($k, $data)=each($positions)) {
			$line 		= substr($phpdoc, $lastpos, ($data["pos"]-$lastpos));
			$value 		= trim(substr($line, strlen($lasttag)));
			$tags[] 	= array ("tag"	=> $lasttag, "value"	=> $value );
			
			$lastpos	= $data["pos"];
			$lasttag	= $data["tag"];
		}
		
		$line 	= substr($phpdoc, $lastpos);
		$value 	= trim(substr($line, strlen($lasttag)));
		$tags[] = array ("tag"	=> $lasttag, "value"	=> $value );

		return $tags;
	} // end func getTags
	
	/**
	* Find the position of the next phpdoc tag.
	* @param	string	$phpdoc	
	* @param	int			$offset
	* @return array		$t					0 => tag, 1 => offset
	* @access	private
	* @see		findTags()
	*/
	function getTagPos($phpdoc, $offset=0) {
		
		$regexp 		= "/@[a-z]{3,9}/";
		$positions	= array();
		
		preg_match_all($regexp, $phpdoc, $regs, PREG_SET_ORDER);
		reset($regs);
		while (list($k, $data)=each($regs)) 
			if (isset($this->PHPDOC_TAGS[$data[0]])) {
				$pos = strpos($phpdoc, $data[0], $offset);
				if ($pos>0 || $data[0]==substr($phpdoc, 0, strlen($data[0])) ) {
					$positions[] = array ("pos" => $pos, "tag" => $data[0]);
					$offset = $pos+1;
				}
			}

		return $positions;
	} // end func getTagPos
	
	/**
	* Extract the value from the given tags and copy the data to the $data array if its an allowed tag
	* @param	array	$tags			array of tags returned by getTags
 	* @param	array	$data			array where the allowed tags and their values are copied to
	* @param	array	$allowed	array of allowed (recognized) tags
	* @return	array	$data
	* @throws	runtime_wrongparam
	* @see	getTags(), analyseVariableParagraphs(), analyseFunctionParagraphs(), analyseClassParagraphs()
	*/
	function analyseTags($tags, $data, $allowed) {
	
		if (!is_array($tags) || !is_array($data) || !is_array($allowed)) {
			$this->err = new exception_runtime_wrongparam("Illegal function call", 1, __FILE__, __LINE__, $this->AUTO_HANDLE_ERROR);
			return $data;
		}

		reset($tags);
		while (list($k, $tag)=each($tags)) {	
		
			$tagname = $tag["tag"];
			if (!isset($allowed[$tagname])) {
				$data["ignored"][] = $tag;
				continue;
			}
			
			switch ($tagname) {
				
				case "@copyright":
				case "@author": 
					$data[substr($tagname, 1)] = $tag["value"];
					break;
					
				case "@var":
					$regexp = sprintf("/%s%s(%s)%s%s%s(.*)/",
															$this->C_VARTYPE,
															$this->C_SPACE_CR_OPTIONAL,
															$this->C_OBJECTNAME_OPTIONAL,
															$this->C_SPACE_CR,
															$this->PHP_VARNAME,
															$this->C_SPACE_CR_OPTIONAL
														);
					if (preg_match($regexp, $tag["value"], $regs)) {
						$data["var"] = array (
																		"type"		=> ("object" == $regs[1]) ? $regs[2] : $regs[1],
																		"desc"		=> $regs[3]
																);
					} else 
						$data["var"]["invalid"] = $tag;
					break;
					
				case "@param":
					$found 	= false;
					
					$regexp = sprintf("/%s%s(%s)%s(%s)%s(.*)/",
															$this->C_VARTYPE,
															$this->C_SPACE_CR_OPTIONAL,
															$this->C_OBJECTNAME_OPTIONAL,
															$this->C_SPACE_CR,
															$this->PHP_VARNAME,
															$this->C_SPACE_CR_OPTIONAL
														);
					if (preg_match($regexp, $tag["value"], $regs)) {
						$found = true;
						$data["param"][] = array (
																				"type"		=> ("object" == $regs[1]) ? "object ".$regs[2] : $regs[1],
																				"name"		=> $regs[3],
																				"desc"		=> $regs[4]
																		);
					} 
					
					
					if ("void" == $tag["value"]) {
						$found = true;
						$data["param"][] = array (
																				"type"		=> "void",
																				"name"		=> "",
																				"desc"		=> ""
																		);
					}
					
					if (!$found) 
						$data["ignored"][] = $tag;
						
					break;

				case "@return":
					$found 	= false;
					
					$regexp = sprintf("/%s%s(%s)%s(%s)%s(.*)/",
															$this->C_VARTYPE,
															$this->C_SPACE_CR_OPTIONAL,
															$this->C_OBJECTNAME_OPTIONAL,
															$this->C_SPACE_CR,
															$this->PHP_VARNAME,
															$this->C_SPACE_CR_OPTIONAL
														);
					if (preg_match($regexp, $tag["value"], $regs)) {
						$found = true;
						$data["return"] = array (
																			"type"		=> ("object" == $regs[1]) ? "object ".$regs[2] : $regs[1],
																			"name"		=> $regs[3],
																			"desc"		=> $regs[4]
																		);
					} 
					
					if ("void" == $tag["value"]) {
						$found = true;
						$data["return"] = array(
																			"type"	=> "void",
																			"name"	=> "",
																			"desc"	=> ""
																	);
					}
						
				
					if (!$found) 
						$data["ignored"][] = $tag;
					break;

				case "@see":
					$regexp = sprintf("/%s/", $this->C_SEE_FUNCTION);
					if (preg_match_all($regexp, $tag["value"], $regs, PREG_SET_ORDER)) {
						reset($regs);
						while (list($k, $v)=each($regs)) 
							$data["see"]["functions"][] = substr($v[0], 0, -2);
						$tag["value"]=preg_replace($regexp, "", $tag["value"]);
					} 
					
					$regexp = sprintf("/%s/", $this->C_SEE_VAR);
					if (preg_match_all($regexp, $tag["value"], $regs, PREG_SET_ORDER)) {
						reset($regs);
						while (list($k, $v)=each($regs)) 
							$data["see"]["var"][] = $v[0];
					}
					break;
					
				case "@throws":
					$exceptions = explode(",", $tag["value"]);
					reset($exceptions);
					while (list($k, $exception)=each($exceptions)) 
						$data["throws"][] = trim($exception);
					break;
					
				case "@access":
					$regexp = sprintf("/%s/", $this->C_ACCESS);
					if (preg_match($regexp, $tag["value"], $regs)) 
						$data["access"] = $tag["value"];
					else 
						$data["ignored"][] = $tag;
					break;
					
				case "@version":
				case "@since":
					$data[substr($tagname, 1)] = $tag["value"];
					break;
					
				case "@brother":
				case "@sister":
					$data["brother"] = $tag["value"];
					break;

				case "@abstract":		
				case "@static":			
					$data[substr($tagname, 1)] = true;
					break;
					
				default:
					$data["ignored"][] = $tag;
					break;
			}	
			
		}	 	

		return $data;		
	} // end func analyseTags
	
} // end class phpdoc_analyser
?>