<?php
/**
* Extract class variables and their documentation from phpcode
* @author   Ulf Wendel <ulf@redsys.de>
* @version  0.2
*/
class PhpdocVariableParser extends PhpdocParserObject {

	/**
	* Array holding all informations about the variables found in the php code
	*
	* The array is indexed by the variable name. $emptyVariable shows
	* you the internal structure.
	*
	* @var		array	$variables
	* @see		emptyVariable
	*/
	var $variables = array();
	
	/**
	* Array with default values of a variable
	* 
	* @var	array	$emptyVariable
	* @see	variables, analyseVariableParagraph()
	*/
	var $emptyVariable = array(
															"name"					=> "",
															"shortdesc"			=> "",
															"desc"					=> "",
															"var"						=> array(),
															"access"				=> "",
															"see"						=> array(),
															"version"				=> "",
															"since"					=> "",
															"abstract"			=> "",
															"static"				=> "",
															"ignored"				=> array(),
															"undocumented"	=> ""
												);

	/**
	* Array of tags that are allowed in front of the var keyword
	* @var	array	$variableTags
	* @see	analyseVariableParagraph()
	*/
	var $variableTags = array(
															"@var"			=> true,
															"@see"				=> true,
															"@access"			=> true,
															"@version"		=> true,
															"@since"			=> true,
															"@abstract"		=> true,
															"@static"			=> true
											);												
	/**
	* Just calls Setup().
	* @see	Setup()
	*/
	function PhpdocVariableParser() {
		$this->Setup();
	} // end constructor
	
	/**
	* Looks for class variables
	* @param	string	$phpcode
	* @return	array		$variables
	* @access	public
	*/
  function analyseVariables($phpcode) {
		$this->variables = array();
		
		$paragraphs = $this->getPhpdocParagraphs($phpcode, "variables");
		reset($paragraphs["variables"]);
		while (list($k, $paragraph)=each($paragraphs["variables"]))
			$this->analyseVariableParagraph($paragraph);
			
		$this->addUndocumentedVariables($phpcode);		
		return $this->variables;
	} // end func analyseVariables
	
	/**
	* Analyses a single phpdoc paragraph describe one variable
	* @param	string	$phpdoc
	* @return	array
	* @access	private
	*/
	
	function analyseVariableParagraph($paragraph) {
		
		$variable = $this->emptyVariable;

		$name = substr($paragraph["regs"][2], 1);
		$variable["name"] = $name;
		
		list($shortdesc, $fulldesc) = $this->getDescription($paragraph["phpdoc"]);
		$variable["shortdesc"]	= $shortdesc;
		$variable["desc"]				= $fulldesc;
		
		$tags = $this->getTags($paragraph["phpdoc"]);
		$variable = $this->analyseTags($tags, $variable, $this->variableTags);

		$variable["undocumented"] = false;
		$this->variables[$name] = $variable;
		
	} // end func analyseVariable
	
	/**
	* Adds undocumented Variables to the list of documented variables
	* @param	string	$phpcode
	* @access	private
	*/
	function addUndocumentedVariables($phpcode) {
		
		$regexp = "/".substr($this->PHP_VAR, 2);
		preg_match_all($regexp, $phpcode, $paragraphs, PREG_SET_ORDER);
		
		reset($paragraphs);
		while (list($k, $paragraph)=each($paragraphs)) {			
			$variable = substr($paragraph[2], 1);
			if (!isset($this->variables[$variable])) {
				$this->variables[$variable] = $this->emptyVariable;
				$this->variables[$variable]["name"] = $variable;
				$this->variables[$variable]["undocumented"] = true;
			}
		}
		
	} // end func addUndocumentedVariables
	
} // end class PhpdocVariableParser
?>